unit fPtSel;
{ Allows patient selection using various pt lists.  Allows display & processing of alerts. }

{$OPTIMIZATION OFF}                              // REMOVE AFTER UNIT IS DEBUGGED

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ORCtrls, ExtCtrls, ORFn, ORDtTmRng, Gauges, Menus;

type
  TfrmPtSel = class(TForm)
    pnlPtSel: TORAutoPanel;
    cboPatient: TORComboBox;
    lblPatient: TLabel;
    cmdOK: TButton;
    cmdCancel: TButton;
    pnlNotifications: TORAutoPanel;
    cmdProcessInfo: TButton;
    cmdProcessAll: TButton;
    cmdProcessSelected: TButton;
    cmdForward: TButton;
    lstNotifications: TORListBox;
    sptVert: TSplitter;
    cmdSaveList: TButton;
    pnlDivide: TORAutoPanel;
    lblNotifications: TLabel;
    ggeInfo: TGauge;
    cmdDeleteSelected: TButton;
    popNotifications: TPopupMenu;
    mnuProcessInfo: TMenuItem;
    mnuProcessAll: TMenuItem;
    mnuProcessSelected: TMenuItem;
    N1: TMenuItem;
    mnuDeleteSelected: TMenuItem;
    mnuForwardSelected: TMenuItem;
    procedure cmdOKClick(Sender: TObject);
    procedure cmdCancelClick(Sender: TObject);
    procedure cboPatientChange(Sender: TObject);
    procedure cboPatientKeyPause(Sender: TObject);
    procedure cboPatientMouseClick(Sender: TObject);
    procedure cboPatientEnter(Sender: TObject);
    procedure cboPatientExit(Sender: TObject);
    procedure cboPatientNeedData(Sender: TObject; const StartFrom: string;
      Direction, InsertAt: Integer);
    procedure cboPatientDblClick(Sender: TObject);
    procedure cmdProcessSelectedClick(Sender: TObject);
    procedure cmdSaveListClick(Sender: TObject);
    procedure cmdProcessInfoClick(Sender: TObject);
    procedure cmdProcessAllClick(Sender: TObject);
    procedure lstNotificationsDblClick(Sender: TObject);
    procedure cmdForwardClick(Sender: TObject);
    procedure cmdDeleteSelectedClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure pnlPtSelResize(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    FDfltSrc: string;
    procedure AdjustFormSize(ShowNotif: Boolean; FontSize: Integer);
    procedure ClearIDInfo;
    procedure ShowIDInfo;
    procedure SetCaptionTop;
    procedure SetPtListTop(IEN: Int64);
    procedure RPLDisplay;
  public
    procedure Loaded; override;
  end;

procedure SelectPatient(ShowNotif: Boolean; FontSize: Integer);

var
  frmPtSel: TfrmPtSel;
  IsRPL, RPLJob: string;                         // RPLJob stores server $J job number of RPL pt. list.
  RPLProblem: boolean;                           // Allows close of form if there's an RPL problem.

implementation

{$R *.DFM}

uses rCore, uCore, fPtSens, fPtSelDemog, fPtSelOptns; {fNotiFwd,}

const
  TX_WARN_SIMREC = CRLF + 'CAUTION!! Multiple patient records exist with this last name' +
                   CRLF + 'and last 4 social security. Are you sure this is your patient?';
  TC_WARN_SIMREC = 'Similar Patient Records';
  TX_DGSR_ERR    = 'Unable to perform sensitive record checks';
  TC_DGSR_ERR    = 'Error';
  TC_DGSR_SHOW   = 'Restricted Record';
  TC_DGSR_DENY   = 'Access Denied';
  TX_DGSR_YESNO  = CRLF + 'Do you want to continue processing this patient record?';

procedure SelectPatient(ShowNotif: Boolean; FontSize: Integer);
{ displays patient selection dialog (with optional notifications), updates Patient object }
var
  frmPtSel: TfrmPtSel;
begin
  frmPtSel := TfrmPtSel.Create(Application);
  RPLProblem := false;
  try
    with frmPtSel do
    begin
      AdjustFormSize(ShowNotif, FontSize);           // Set initial form size
      FDfltSrc := DfltPtList;
      if (IsRPL = '1') then                          // Deal with restricted patient list users.
        FDfltSrc := '';
      frmPtSelOptns.SetDefaultPtList(FDfltSrc);
      if RPLProblem then
         begin
          frmPtSel.Release;
          exit;
        end;
      Notifications.Clear;
      LoadNotifications(lstNotifications.Items);
      ClearIDInfo;
      if (IsRPL = '1') then                          // Deal with restricted patient list users.
        RPLDisplay;                                  // Removes unnecessary components from view.
      ShowModal;
    end;
  finally
    frmPtSel.Release;
  end;
end;

procedure TfrmPtSel.AdjustFormSize(ShowNotif: Boolean; FontSize: Integer);
{ Adjusts the initial size of the form based on the font used & if notifications should show. }
var
  W,H: Integer;
begin
  // Assume MS Sans Serif font, use the ratios of font heights & widths relative to the
  //   designed size (8 pt) to figure out how big the form should be.
  W := pnlPtSel.Width; H := pnlPtSel.Height;
  case FontSize of
   8: { No need to change, 8pt is the designed size. } ;
  10: begin W := Round(W*( 7/6)+1); H := Round(H*(16/13)+1); end;
  12: begin W := Round(W*( 9/6)+1); H := Round(H*(20/13)+1); end;
  14: begin W := Round(W*(10/6)+1); H := Round(H*(24/13)+1); end;
  18: begin W := Round(W*(13/6)+1); H := Round(H*(29/13)+1); end;
  end;
  pnlPtSel.Width  := W;  pnlPtSel.Height := H;
  ClientWidth     := W;  ClientHeight    := H;
  if FontSize in [8, 10, 12, 14, 18] then
  begin
    Font.Size := FontSize;
    frmPtSelOptns.Font.Size := FontSize;
    frmPtSelDemog.Font.Size := FontSize;
    frmPtSelDemog.lblPtName.Font.Size := FontSize;             // Have to do bolded controls individually.
  end;
  // Make the form bigger (57%) to show notifications and show notification controls.
  if ShowNotif then
  begin
    ClientHeight := Round(H * 1.57);
    pnlDivide.Height := lblNotifications.Height + 4;
    pnlDivide.Visible := True;
    lstNotifications.Visible := True;
    pnlNotifications.Visible := True;
    pnlPtSel.BevelOuter := bvRaised;
  end;
end;

procedure TfrmPtSel.SetCaptionTop;
{ Show patient list name, set top list to 'Select ...' if appropriate. }
var
  x: string;
begin
  x := '';
  lblPatient.Caption := 'Patients';
  if (not User.IsReportsOnly) then
  begin
  case frmPtSelOptns.SrcType of
  TAG_SRC_DFLT: lblPatient.Caption := 'Patients (' + FDfltSrc + ')';
  TAG_SRC_PROV: x := 'Provider';
  TAG_SRC_TEAM: x := 'Team';
  TAG_SRC_SPEC: x := 'Specialty';
  TAG_SRC_CLIN: x := 'Clinic';
  TAG_SRC_WARD: x := 'Ward';
  TAG_SRC_ALL:  { Nothing };
  end; // case stmt
  end; // begin
  if Length(x) > 0 then with cboPatient do
  begin
    RedrawSuspend(Handle);
    ClearIDInfo;
    ClearTop;
    Text := '';
    Items.Add('^Select a ' + x + '...');
    Items.Add(LLS_LINE);
    Items.Add(LLS_SPACE);
    cboPatient.InitLongList('');
    RedrawActivate(cboPatient.Handle);
  end;
end;

{ List Source events: }

procedure TfrmPtSel.SetPtListTop(IEN: Int64);
{ Sets top items in patient list according to list source type and optional list source IEN. }
var
  NewTopList: string;
  //FirstDate, LastDate: TFMDateTime;
  FirstDate, LastDate: string;
begin
  IsRPL := User.IsRPL;
  if (IsRPL = '') then // First piece in ^VA(200,.101) should always be set (to 1 or 0).
    begin
      InfoBox('Patient selection list flag not set.', 'Incomplete User Information', MB_OK);
      RPLProblem := true;
      exit;
    end;
  //FirstDate := 0; LastDate := 0; // Not req'd, but eliminates hint.
  // Assign list box TabPosition, Pieces properties according to type of list to be displayed.
  // (Always use Piece "2" as the first in the list to assure display of patient's name.)
  cboPatient.pieces := '2,3'; // This line and next: defaults set - exceptions modifield next.
  cboPatient.tabPositions := '20,28';
  if ((frmPtSelOptns.SrcType = TAG_SRC_DFLT) and (FDfltSrc = 'Combination')) then
    begin
      cboPatient.pieces := '2,3,4,5';
      cboPatient.tabPositions := '20,28,35';
    end;
  if ((frmPtSelOptns.SrcType = TAG_SRC_DFLT) and
      ((FDfltSrc = 'Ward') or (frmPtSelOptns.SrcType = TAG_SRC_WARD))) then
    cboPatient.pieces := '2,4';

  NewTopList := IntToStr(frmPtSelOptns.SrcType) + U + IntToStr(IEN); // Default setting.
  if (frmPtSelOptns.SrcType = TAG_SRC_CLIN) then with frmPtSelOptns.cboDateRange do
    begin
      if ItemID = '' then Exit; // Need both clinic & date range.
      //FirstDate := MakeFMDateTime(Piece(ItemID, ';', 1));
      //LastDate  := MakeFMDateTime(Piece(ItemID, ';', 2));
      FirstDate := Piece(ItemID, ';', 1);
      LastDate  := Piece(ItemID, ';', 2);
      NewTopList := IntToStr(frmPtSelOptns.SrcType) + U + IntToStr(IEN) + U + ItemID; // Modified for clinics.
    end;
  if NewTopList = frmPtSelOptns.LastTopList then Exit; // Only continue if new top list.
  frmPtSelOptns.LastTopList := NewTopList;
  RedrawSuspend(cboPatient.Handle);
  ClearIDInfo;
  cboPatient.ClearTop;
  cboPatient.Text := '';
  if (IsRPL = '1') then                                // Deal with restricted patient list users.
    begin
      RPLJob := MakeRPLPtList(User.RPLList);           // MakeRPLPtList is in rCore, writes global "B" x-ref list.
      if (RPLJob = '') then
        begin
          InfoBox('Assignment of valid Patient/Team List Needed.', 'Unable to build Patient List', MB_OK);
          RPLProblem := true;
          exit;
        end;
    end
  else
    begin
      case frmPtSelOptns.SrcType of
      TAG_SRC_DFLT: ListPtByDflt(cboPatient.Items);
      TAG_SRC_PROV: ListPtByProvider(cboPatient.Items, IEN);
      TAG_SRC_TEAM: ListPtByTeam(cboPatient.Items, IEN);
      TAG_SRC_SPEC: ListPtBySpecialty(cboPatient.Items, IEN);
      TAG_SRC_CLIN: ListPtByClinic(cboPatient.Items, frmPtSelOptns.cboList.ItemIEN, FirstDate, LastDate);
      TAG_SRC_WARD: ListPtByWard(cboPatient.Items, IEN);
      TAG_SRC_ALL:  ListPtTop(cboPatient.Items);
      end;
    end;
  if frmPtSelOptns.cboList.Visible then
    lblPatient.Caption := 'Patients (' + frmPtSelOptns.cboList.Text + ')';
  with cboPatient do if ShortCount > 0 then
    begin
      Items.Add(LLS_LINE);
      Items.Add(LLS_SPACE);
    end;
  cboPatient.InitLongList('');
  RedrawActivate(cboPatient.Handle);
end;

{ Patient Select events: }

procedure TfrmPtSel.cboPatientEnter(Sender: TObject);
begin
  cmdOK.Default := True;
end;

procedure TfrmPtSel.cboPatientExit(Sender: TObject);
begin
  cmdOK.Default := False;
end;

procedure TfrmPtSel.cboPatientChange(Sender: TObject);

    procedure ShowMatchingPatients;  
    begin
      with cboPatient do
        begin
          ClearIDInfo;
          if ShortCount > 0 then
            begin
              if ShortCount = 1 then
                begin
                  ItemIndex := 0;
                  ShowIDInfo;
                end;
              Items.Add(LLS_LINE);
              Items.Add(LLS_SPACE);
            end;
          InitLongList('');
        end;
    end;

begin
  with cboPatient do
    if frmPtSelOptns.IsLast5(Text) then
      begin
        if (IsRPL = '1') then
          ListPtByRPLLast5(Items, Text)
        else
          ListPtByLast5(Items, Text);
        ShowMatchingPatients;
      end
    else if frmPtSelOptns.IsFullSSN(Text) then
      begin
        if (IsRPL = '1') then
           ListPtByRPLFullSSN(Items, Text)
        else
           ListPtByFullSSN(Items, Text);
        ShowMatchingPatients;
      end;
end;

procedure TfrmPtSel.cboPatientKeyPause(Sender: TObject);
begin
  if Length(cboPatient.ItemID) > 0 then  //*DFN*
  begin
    ShowIDInfo;
  end else
  begin
    ClearIDInfo;
  end;
end;

procedure TfrmPtSel.cboPatientMouseClick(Sender: TObject);
begin
  if Length(cboPatient.ItemID) > 0 then   //*DFN*
  begin
    ShowIDInfo;
  end else
  begin
    ClearIDInfo;
  end;
end;

procedure TfrmPtSel.cboPatientDblClick(Sender: TObject);
begin
  if Length(cboPatient.ItemID) > 0 then cmdOKClick(Self);  //*DFN*
end;

procedure TfrmPtSel.cboPatientNeedData(Sender: TObject; const StartFrom: string;
  Direction, InsertAt: Integer);
begin
  if (IsRPL  = '1') then // Restricted patient lists uses different feed for long list box:
    cboPatient.ForDataUse(ReadRPLPtList(RPLJob, StartFrom, Direction))
  else
    cboPatient.ForDataUse(SubSetOfPatients(StartFrom, Direction));
end;

procedure TfrmPtSel.ClearIDInfo;
begin
  frmPtSelDemog.ClearIDInfo;
end;

procedure TfrmPtSel.ShowIDInfo;
begin
  frmPtSelDemog.ShowDemog(cboPatient.ItemID);
end;

{ Command Button events: }

procedure TfrmPtSel.cmdOKClick(Sender: TObject);
{ Checks for restrictions on the selected patient and sets up the Patient object. }
const
  DLG_CANCEL = False;
  DGSR_FAIL = -1;
  DGSR_NONE =  0;
  DGSR_SHOW =  1;
  DGSR_ASK  =  2;
  DGSR_DENY =  3;
var
  NewDFN, AMsg: string;  //*DFN*
  AccessStatus: Integer;
  DateDied: TFMDateTime;
begin
  if not (Length(cboPatient.ItemID) > 0) then  //*DFN*
  begin
    InfoBox('A patient has not been selected.', 'No Patient Selected', MB_OK);
    Exit;
  end;
  NewDFN := cboPatient.ItemID;  //*DFN*
  if SimilarRecordsFound(NewDFN, AMsg) and
    (InfoBox(AMsg, TC_WARN_SIMREC, MB_YESNO or MB_ICONWARNING) <> IDYES) then Exit;
  CheckSensitiveRecordAccess(NewDFN, AccessStatus, AMsg);
  case AccessStatus of
  DGSR_FAIL: begin
               InfoBox(TX_DGSR_ERR, TC_DGSR_ERR, MB_OK);
               Exit;
             end;
  DGSR_NONE: { Nothing - allow access to the patient. };
  DGSR_SHOW: InfoBox(AMsg, TC_DGSR_SHOW, MB_OK);
  DGSR_ASK:  if InfoBox(AMsg + TX_DGSR_YESNO, TC_DGSR_SHOW, MB_YESNO or MB_ICONWARNING or
               MB_DEFBUTTON2) = IDYES then LogSensitiveRecordAccess(NewDFN) else Exit;
  else       begin
               InfoBox(AMsg, TC_DGSR_DENY, MB_OK);
               Exit;
             end;
  end;
  DateDied := DateOfDeath(NewDFN);
  if (DateDied > 0) and
    (InfoBox('This patient died ' + FormatFMDateTime('mmm dd,yyyy hh:nn', DateDied) + CRLF +
     'Do you wish to continue?', 'Deceased Patient', MB_YESNO or MB_DEFBUTTON2) = ID_NO)
     then Exit;
  if NewDFN <> Patient.DFN then
  begin
    Patient.DFN := NewDFN;     // The patient object in uCore must have been created already!
    Encounter.Clear;
    Changes.Clear;             // An earlier call to ReviewChanges should have cleared this.
    if (frmPtSelOptns.SrcType = TAG_SRC_CLIN) and (frmPtSelOptns.cboList.ItemIEN > 0) and
      (Length(Piece(cboPatient.Items[cboPatient.ItemIndex], U, 5)) > 0) then // Clinics.
    begin
      Encounter.Location := frmPtSelOptns.cboList.ItemIEN;
      with cboPatient do Encounter.DateTime := MakeFMDateTime(Piece(Items[ItemIndex], U, 4));
    end
    else if (frmPtSelOptns.SrcType = TAG_SRC_DFLT) and (DfltPtListSrc = 'C') and
            (Length(Piece(cboPatient.Items[cboPatient.ItemIndex], U, 5)) > 0) then
         with cboPatient do  {*KCM*} // "Default" is a clinic.
    begin
      Encounter.Location := StrToIntDef(Piece(Items[ItemIndex], U, 5), 0);
      Encounter.DateTime := MakeFMDateTime(Piece(Items[ItemIndex], U, 4));
    end
    else if ((frmPtSelOptns.SrcType = TAG_SRC_DFLT) and
             (copy(Piece(cboPatient.Items[cboPatient.ItemIndex], U, 3), 1, 2) = 'Cl')) and
             (Length(Piece(cboPatient.Items[cboPatient.ItemIndex], U, 4)) > 0) then
         with cboPatient do {PKS - 2/2001} // "Default" combination, clinic pt.
    begin
      Encounter.Location := StrToIntDef(Piece(Items[ItemIndex], U, 7), 0);
      Encounter.DateTime := MakeFMDateTime(Piece(Items[ItemIndex], U, 8));
    end
    else if Patient.Inpatient then // Everything else:
    begin
      Encounter.Inpatient := True;
      Encounter.Location := Patient.Location;
      Encounter.DateTime := Patient.AdmitTime;
      Encounter.VisitCategory := 'H';
    end;
    if User.IsProvider then Encounter.Provider := User.DUZ;
  end;
  Close;
end;

procedure TfrmPtSel.cmdCancelClick(Sender: TObject);
begin
  // Leave Patient object unchanged
  Close;
end;

procedure TfrmPtSel.cmdProcessSelectedClick(Sender: TObject);
var
  AFollowUp, i, infocount: Integer;
  enableclose: boolean;
  ADFN, x: string;  //*DFN*
begin
  enableclose := false;
  with lstNotifications do
  begin
    if SelCount <= 0 then Exit;

    // Count information-only selections for gauge
    infocount := 0;
    for i:= 0 to Items.count - 1 do if Selected[i] then
      if (Piece(items[i], U, 4) = 'I') then Inc(infocount);

    if infocount >= 1 then
    begin
      ggeInfo.Visible := true; (*BOB*)
      ggeInfo.MaxValue := infocount;
    end;

    for i := 0 to Items.Count - 1 do if Selected[i] then
    begin
      if (Piece(items[i], U, 4) = 'I') then
    // If fourth piece is 'I' delete the information only alert.
    //'   HOOD,ROBI (H2591): New order(s) placed.^OR,16,50;1311;2980626.100756^1^I'
        begin
          ggeInfo.Progress := ggeInfo.Progress + 1;
          DeleteAlert(Piece(Items[i], U, 2));
        end
      else if Piece(Piece(Items[i], U, 2), ',', 1) = 'OR' then
    //'   HOOD,ROBI (H2591): New order(s) placed.^OR,16,50;1311;2980626.100756^2^ '
        begin
          ADFN := Piece(Piece(Items[i], U, 2), ',', 2);  //*DFN*
          AFollowUp := StrToIntDef(Piece(Piece(Piece(Items[i], U, 2), ';', 1), ',', 3), 0);
          Notifications.Add(ADFN, AFollowUp, Items[i]);
          enableclose := true;
        end
      else if Copy(Piece(Items[i], U, 2), 1, 6) = 'TIUERR' then
        InfoBox(Piece(Items[i], U, 1) + #13#10#13#10 +
           'The CPRS GUI can not yet process this type of alert.  Please use List Manager.',
           'Unable to Process Alert', MB_OK)
      else if Copy(Piece(Items[i], U, 2), 1, 3) = 'TIU' then
    //'   ABC,PATIE (A4321): Addendum available for signature.^TIU4012;1329;2980626.115947^2^ '
        begin
          x := GetTIUAlertInfo(Piece(Items[i], U, 2));
          if Piece(x, U, 2) <> '' then
            begin
              ADFN := Piece(x, U, 2);  //*DFN*
              AFollowUp := StrToIntDef(Piece(Piece(x, U, 3), ';', 1), 0);
              Notifications.Add(ADFN, AFollowUp, Items[i] + Piece(x, U, 3));
              enableclose := true;
            end
          else
            DeleteAlert(Piece(Items[i], U, 2));
        end;
    end;
    if enableclose = true then
      Close
    else
      begin
        ggeInfo.Visible := False;
        // Update notification list:
        lstNotifications.Clear;
        LoadNotifications(lstNotifications.Items);
      end;
  end;
end;

procedure TfrmPtSel.cmdSaveListClick(Sender: TObject);
begin
  frmPtSelOptns.cmdSaveListClick(Sender);
end;

procedure TfrmPtSel.cmdProcessInfoClick(Sender: TObject);
  // Select and process all items that are information only in the lstNotifications list box.
var
  i: integer;
begin
  for i := 0 to lstNotifications.Items.Count-1 do
    lstNotifications.Selected[i] := (Piece(lstNotifications.Items[i], U, 4) = 'I');
  cmdProcessSelectedClick(Self);
end;

procedure TfrmPtSel.cmdProcessAllClick(Sender: TObject);
var
  i: integer;
begin
  for i := 0 to lstNotifications.Items.Count-1 do
    lstNotifications.Selected[i] := True;
  cmdProcessSelectedClick(Self);
end;

procedure TfrmPtSel.lstNotificationsDblClick(Sender: TObject);
begin
  cmdProcessSelectedClick(Self);
end;

procedure TfrmPtSel.cmdForwardClick(Sender: TObject);
{var
  Alerts: TStringList;
  //fupresult : boolean;
  i: integer;}
begin
{  Alerts := TStringList.Create;
  try
    with lstNotifications do
      begin
        if SelCount <= 0 then Exit;
        for i := 0 to Items.Count - 1 do
          if Selected[i] then
            Alerts.Add(Items[i]);
      end;
    ForwardAlerts(Alerts);
  finally
    Alerts.Free;
  end;   }
end;

procedure TfrmPtSel.cmdDeleteSelectedClick(Sender: TObject);
{var
  i: integer;}
begin
{  with lstNotifications do
    begin
      if SelCount <= 0 then Exit;
      for i := 0 to Items.Count - 1 do
        if Selected[i] then
          DeleteAlert(Piece(Items[i], U, 2));
    end;
  Notifications.Clear;
  LoadNotifications(lstNotifications.Items); }
end;

procedure TfrmPtSel.FormDestroy(Sender: TObject);
begin
  //frmPtSelDemog.Free;
  //frmPtSelOptns.Free;
end;

procedure TfrmPtSel.pnlPtSelResize(Sender: TObject);
begin
  frmPtSelDemog.Left := cboPatient.Left + cboPatient.Width + 9;
  frmPtSelDemog.Width := pnlPtSel.Width - frmPtSelDemog.Left - 2;
  frmPtSelOptns.Width := cboPatient.Left-8;
end;

procedure TfrmPtSel.Loaded;
begin
  inherited;
// This needs to be in Loaded rather than FormCreate or the TORAutoPanel resize logic breaks.
  frmPtSelDemog := TfrmPtSelDemog.Create(Self);  // Was application - kcm
  with frmPtSelDemog do
  begin
    parent := pnlPtSel;
    Top := 80;
    Left := cboPatient.Left + cboPatient.Width + 9;
    Width := pnlPtSel.Width - Left - 2;
    Show;
  end;

  frmPtSelOptns := TfrmPtSelOptns.Create(Self);  // Was application - kcm
  with frmPtSelOptns do
  begin
    parent := pnlPtSel;
    Top := 4;
    Left := 4;
    Width := cboPatient.Left-8;
    SetCaptionTopProc := SetCaptionTop;
    SetPtListTopProc  := SetPtListTop;
    if RPLProblem then
      exit;
    TabOrder := 3;
    cmdSaveList.TabOrder := 4;
    Show;
  end;
end;

procedure TfrmPtSel.RPLDisplay;
begin

// Make unneeded components invisible:
cmdSaveList.visible := false;
frmPtSelOptns.visible := false;

end;

procedure TfrmPtSel.FormClose(Sender: TObject; var Action: TCloseAction);
begin

if (IsRPL = '1') then                          // Deal with restricted patient list users.
  KillRPLPtList(RPLJob);                       // Kills server global data each time.
                                               // (Global created by MakeRPLPtList in rCore.)
end;

end.

