unit uSignItems;

interface

uses
  SysUtils, Windows, Classes, Graphics, Controls, StdCtrls, CheckLst, ORClasses, ORCtrls;

type
  TSigItemType = (siServiceConnected, siAgentOrange, siIonizingRadiation,
                  siEnvironmentalContaminants, siMST, siHeadNeckCancer);

  TSigItemTagInfo = record
    SigType: TSigItemType;
    Index: integer;
  end;

  TSigItems = class(TComponent)
  private
    FBuilding: boolean;
    FStsCount: integer;
    FItems: TORStringList;
    FOldDrawItemEvent: TDrawItemEvent;
    Fcb: TList;
    Flb: TCustomListBox;
    FLastSigItem: TSigItemType;
    FLastValidX: integer;
    FValidGap: integer;
    FDy: integer;
    FAllCheck: array[TSigItemType] of boolean;
    FAllCatCheck: boolean;
    FcbX: array[TSigItemType] of integer;
    function TagInfo(ASigType: TSigItemType; AIndex: integer): TSigItemTagInfo;
    function ItemToTag(Info: TSigItemTagInfo): integer;
    function TagToItem(ATag: integer): TSigItemTagInfo;
    procedure cbClicked(Sender: TObject);
    procedure lbDrawItem(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Add(ItemType: Integer; const ID: string; Index: integer);
    procedure Remove(ItemType: integer; const ID: string);
    procedure ResetOrders;
    procedure Clear;
    function UpdateListBox(lb: TCustomListBox): boolean;
    procedure EnableSettings(Index: integer; Checked: boolean);
    function OK2SaveSettings: boolean;
    procedure SaveSettings;
  end;

function SigItems: TSigItems;
function SigItemHeight: integer;

const
  TC_CoPay_Error = 'Unanswered Outpatient Medications Questions';
  TX_CoPay_Error = 'One or more Outpatient Medications Related Questions have not been answered.';

implementation

uses
  ORFn, ORNet, uConst, TRPCB, rOrders, rPCE;

type
  ItemStatus = (isNA, isChecked, isUnchecked, isUnknown);
  SigDescType = (sdShort, sdLong);

const
  SigItemDesc: array[TSigItemType, SigDescType] of string =
        { siServiceConnected          } (('SC',  'Service Connected Condition'),
        { siAgentOrange               }  ('AO',  'Agent Orange Exposure'),
        { siIonizingRadiation         }  ('IR',  'Ionizing Radiation Exposure'),
        { siEnvironmentalContaminants }  ('EC',  'Environmental Contaminants'),
        { siMST                       }  ('MST', 'MST'), //'Military Sexual Trauma'
        { siHeadNeckCancer            }  ('HNC', 'Head and/or Neck Cancer'));

  StsChar: array[ItemStatus] of char =
           { isNA        } ('N',
           { isChecked   }  'C',
           { isUnchecked }  'U',
           { isUnknown   }  '?');

  ColIdx = 30000;
  AllIdx = 31000;

var
  uSingletonFlag: boolean = FALSE;
  uSigItems: TSigItems = nil;
  FlagCount: integer;
  BaseFlags: string;

function SigItems: TSigItems;
begin
  if not assigned(uSigItems) then
  begin
    uSingletonFlag := TRUE;
    try
      uSigItems := TSigItems.Create(nil);
    finally
      uSingletonFlag := FALSE;
    end;
  end;
  Result := uSigItems;
end;

function SigItemHeight: integer;
begin
  Result := MainFontHeight + 2;
end;

{ TSigItems }
{
FItems Layout:
1         2               3                            4                      5
OrderID ^ ListBox Index ^ RPC Call was Made (0 or 1) ^ Settings by char pos ^ Disabled Flag
}

procedure TSigItems.Add(ItemType: Integer; const ID: string; Index: integer);
var
  idx: integer;

begin
  if ItemType = CH_ORD then
  begin
    idx := FItems.IndexOfPiece(ID);
    if idx < 0 then
      idx := FItems.Add(ID);
    FItems.SetStrPiece(idx, 2, IntToStr(Index));
  end;
end;

procedure TSigItems.Clear;
begin
  FItems.Clear;
  Fcb.Clear;
end;

constructor TSigItems.Create(AOwner: TComponent);
begin
  if not uSingletonFlag then
    raise Exception.Create('Only one instance of TSigItems allowed');
  inherited Create(AOwner);
  FItems := TORStringList.Create;
  Fcb := TList.Create;
  FLastSigItem := high(TSigItemType);
  if not HNCOK then
  begin
    FLastSigItem := pred(FLastSigItem);
    dec(FlagCount);
    BaseFlags := StringOfChar(StsChar[isNA], FlagCount);
  end;
end;

destructor TSigItems.Destroy;
begin
  FreeAndNil(FItems);
  FreeAndNil(Fcb);
  inherited;
end;

procedure TSigItems.Remove(ItemType: integer; const ID: string);
var
  idx: integer;

begin
  if ItemType = CH_ORD then
  begin
    idx := FItems.IndexOfPiece(ID);
    if idx >= 0 then
      FItems.Delete(idx);
  end;
end;

procedure TSigItems.ResetOrders; // Resets ListBox positions, to avoid old data messing things up
var
  i: integer;

begin
  for i := 0 to FItems.Count-1 do
    FItems.SetStrPiece(i, 2, '-1');
end;

function TSigItems.ItemToTag(Info: TSigItemTagInfo): integer;
begin
  if Info.Index < 0 then
    Result := 0
  else
    Result := (Info.Index*FlagCount) + ord(Info.SigType) + 1;
end;

function TSigItems.TagInfo(ASigType: TSigItemType;
  AIndex: integer): TSigItemTagInfo;
begin
  Result.SigType := ASigType;
  Result.Index := AIndex;
end;

function TSigItems.TagToItem(ATag: integer): TSigItemTagInfo;
begin
  if ATag <= 0 then
  begin
    Result.Index := -1;
    Result.SigType := TSigItemType(0);
  end
  else
  begin
    dec(ATag);
    Result.SigType := TSigItemType(ATag mod FlagCount);
    Result.Index := ATag div FlagCount;
  end;
end;

type
  TExposedListBox = class(TCustomListBox)
  public
    property OnDrawItem;
  end;

function TSigItems.UpdateListBox(lb: TCustomListBox): boolean;
const
  cbWidth = 13;
  cbHeight = 13;
  btnGap = 2;
  AllTxt = 'All';

var
  cb: TORCheckBox;
  btn: TButton;
  lbl: TLabel;
  prnt: TWinControl;
  ownr: TComponent;
  FirstValidItem: TSigItemType;
  x, y, MaxX, i, btnW, j, dx, ht, idx: integer;
  s, id, Code, cType, Flags: string;
  StsCode: char;
  si: TSigItemType;
  sts, StsIdx: ItemStatus;
  StsUsed: array[TSigItemType] of boolean;

  function CreateCB(AParent: TWinControl): TORCheckBox;
  begin
    Result := TORCheckBox.Create(ownr);
    Result.Parent := AParent;
    Result.Height := cbHeight;
    Result.Width := cbWidth;
    Result.GrayedStyle := gsBlueQuestionMark;
    Result.GrayedToChecked := FALSE;
    Result.OnClick := cbClicked;
    Fcb.Add(Result);
  end;

begin
  Result := FALSE;
  Fcb.Clear;
  FBuilding := TRUE;
  try
    idx := 0;
    RPCBrokerV.ClearParameters := True;
    for i := 0 to FItems.Count-1 do
    begin
      s := FItems[i];
      if (piece(s, U, 2) <> '-1') and (piece(s, U, 3) <> '1') then
      begin
        with RPCBrokerV do
        begin
          if idx = 0 then
          begin
            Param[1].PType := list;
          end;
          inc(idx);
          Param[1].Mult[IntToStr(idx)] := piece(s, U, 1);
        end;
      end;
    end;
    if idx > 0 then
    begin
      GetCoPay4Orders;
// testing
{
      for i := 0 to RPCBrokerV.Results.Count-1 do
        RPCBrokerV.Results[I] := RPCBrokerV.Results[I] + u + 'SC';

      RPCBrokerV.Results.Clear;
      if RPCBrokerV.Results.Count = 0 then
      begin
        for i := 0 to FItems.Count-1 do
        begin
          RPCBrokerV.Results.Add(piece(FItems[i],U,1) + U + 'NNNNNN');
        end;
        for si := low(TSigItemType) to FLastSigItem do
          if (si = low(TSigItemType)) or  (random(2) = 1) then
            for i := 0 to FItems.Count-1 do
              RPCBrokerV.Results[i] := RPCBrokerV.Results[i] + SigItemDesc[si, sdCode] + U;//; + inttostr(random(3)) + U;
    end;
 }
// testing
      for i := 0 to RPCBrokerV.Results.Count-1 do
      begin
        s := RPCBrokerV.Results[i];
        id := piece(s,U,1);
        idx := FItems.IndexOfPiece(id);
        if idx >= 0 then
        begin
          FItems.SetStrPiece(idx, 3, '1'); // Mark as read from RPC
          j := 2;
          Flags := BaseFlags;
          repeat
            Code := piece(s,U,j);
            if Code <> '' then
            begin
              cType := piece(Code, ';', 1);
              for si := low(TSigItemType) to FLastSigItem do
              begin
                if cType = SigItemDesc[si, sdShort] then
                begin
                  cType := piece(Code, ';', 2);
                  if cType = '0' then
                    sts := isUnchecked
                  else
                  if cType = '1' then
                    sts := isChecked
                  else
                    sts := isUnknown;
                  Flags[ord(si)+1] := StsChar[sts];
                  break;
                end;
              end;
            end;
            inc(j);
          until(Code = '');
          FItems.SetStrPiece(idx, 4, Flags);
        end;
      end;
    end;
    FStsCount := 0;
    for si := low(TSigItemType) to FLastSigItem do
      StsUsed[si] := FALSE;
    for i := 0 to FItems.Count-1 do
    begin
      s := FItems[i];
      if (piece(s,u,2) <> '-1') and (piece(s,u,3) = '1') then
      begin
        s := piece(s, u, 4);
        for si := low(TSigItemType) to FLastSigItem do
          if (not StsUsed[si]) and (s[ord(si)+1] <> StsChar[isNA]) then
          begin
            StsUsed[si] := TRUE;
            inc(FStsCount);
            if FStsCount >= FlagCount then break;
          end;
      end;
      if FStsCount >= FlagCount then break;
    end;
    if FStsCount > 0 then
    begin
      Result := TRUE;
      FirstValidItem := TSigItemType(0);
      for si := low(TSigItemType) to FLastSigItem do
        if StsUsed[si] then
        begin
          FirstValidItem := si;
          break;
        end;
      prnt := lb.Parent;
      ownr := lb.Owner;
      MaxX := lb.ClientWidth;
      btnW := 0;
      for si := low(TSigItemType) to FLastSigItem do
      begin
        j := lb.Canvas.TextWidth(SigItemDesc[si, sdShort]);
        if btnW < j then
          btnW := j;
      end;
      inc(btnW, 8);
      x := MaxX;
      dx := (btnW - cbWidth) div 2;
      for si := FLastSigItem downto low(TSigItemType) do
      begin
        FcbX[si] := x - btnW + dx;
        dec(x, btnW + btnGap);
      end;
      if FStsCount > 1 then
      begin
        FAllCatCheck := FALSE;
        btn := TButton.Create(ownr);
        btn.Parent := prnt;
        btn.Height := 21;
        btn.Width := btnW;
        btn.Caption := AllTxt;
        btn.OnClick := cbClicked;
        btn.Left := FcbX[TSigItemType(0)] + lb.Left - dx + 2 - (FcbX[TSigItemType(1)] - FcbX[TSigItemType(0)]);
        btn.Top := lb.Top - btn.height - 2;
        btn.Tag := AllIdx;
        btn.ShowHint := TRUE;
        btn.Hint := 'Set All Related Entries';
        Fcb.Add(btn);
      end;
      for si := low(TSigItemType) to FLastSigItem do
      begin
        FAllCheck[si] := TRUE;
        btn := TButton.Create(ownr);
        btn.Parent := prnt;
        btn.Height := 21;
        btn.Width := btnW;
        btn.Caption := SigItemDesc[si, sdShort];
        btn.OnClick := cbClicked;
        btn.Left := FcbX[si] + lb.Left - dx + 2;
        btn.Top := lb.Top - btn.height - 2;
        btn.Tag := ColIdx + ord(si);
        btn.ShowHint := TRUE;
        btn.Hint := 'Set all ' + SigItemDesc[si, sdLong];
        btn.Enabled := StsUsed[si];
        Fcb.Add(btn);
      end;
      FValidGap := ((FcbX[succ(TSigItemType(0))] - FcbX[TSigItemType(0)] - cbWidth) div 2) + 1;
      FLastValidX := FcbX[FirstValidItem] - FValidGap;
      lb.ControlStyle := lb.ControlStyle + [csAcceptsControls];
      try
        ht := SigItemHeight;
        FDy := ((ht - cbHeight) div 2);
        y := lb.TopIndex;
        FOldDrawItemEvent := TExposedListBox(lb).OnDrawItem;
        Flb := lb;
        TExposedListBox(lb).OnDrawItem := lbDrawItem;
        lb.FreeNotification(Self);
        for i := 0 to FItems.Count-1 do
        begin
          s := FItems[i];
          if piece(s,u,3) = '1' then
          begin
            idx := StrToIntDef(piece(s,U,2),-1);
            if idx >= 0 then
            begin
              Flags := piece(s,u,4);
              for si := low(TSigItemType) to FLastSigItem do
              begin
                StsCode := Flags[ord(si)+1];
                StsIdx := isNA;
                for sts := low(ItemStatus) to high(ItemStatus) do
                  if StsCode = StsChar[sts] then
                  begin
                    StsIdx := sts;
                    break;
                  end;
                if (StsIdx <> isNA) then
                begin
                  cb := CreateCB(lb);
                  cb.Left := FcbX[si];
                  cb.Top := (ht * (idx - y)) + FDy;
                  cb.Tag := ItemToTag(TagInfo(si, idx));
                  cb.ShowHint := TRUE;
                  cb.Hint := SigItemDesc[si, sdLong];
                  case StsIdx of
                    isChecked:   cb.State := cbChecked;
                    isUnchecked: cb.State := cbUnchecked;
                    else         cb.State := cbGrayed;
                  end;
                end;
              end;
            end;
          end;
        end;
      finally
        lb.ControlStyle := lb.ControlStyle - [csAcceptsControls];
      end;
    end;
  finally
    FBuilding := FALSE;
  end;
end;

procedure TSigItems.cbClicked(Sender: TObject);
var
  i,cnt,p: integer;
  cb: TORCheckBox;
  sType: TSigItemType;
  idx, Flags: string;
  Info: TSigItemTagInfo;
  wc, w: TWinControl;

begin
  if FBuilding then exit;
  wc := TWinControl(Sender);
  if wc.Tag = AllIdx then
  begin
    FAllCatCheck := not FAllCatCheck;
    for sType := low(TSigItemType) to FLastSigItem do
      FAllCheck[sType] := FAllCatCheck;
    cnt := 0;
    for i := 0 to Fcb.Count-1 do
    begin
      w := TWinControl(Fcb[i]);
      if (w <> wc) and (w.Tag >= ColIdx) and (w is TButton) then
      begin
        inc(cnt);
        if w.Enabled then
          TButton(w).Click;
        if cnt >= FlagCount then break;
      end;
    end;
  end
  else
  if wc.Tag >= ColIdx then
  begin
    sType := TSigItemType(wc.Tag - ColIdx);
    FAllCheck[sType] := not FAllCheck[sType];
    for i := 0 to Fcb.Count-1 do
    begin
      w := TWinControl(Fcb[i]);
      if (w.Tag < ColIdx) and (w is TORCheckBox) then
      begin
        if TagToItem(w.Tag).SigType = sType then
          TORCheckBox(w).Checked := FAllCheck[sType];
      end;
    end;
  end
  else
  begin
    cb := TORCheckBox(wc);
    info := TagToItem(cb.Tag);
    if info.Index >= 0 then
    begin
      idx := inttostr(info.Index);
      i := FItems.IndexOfPiece(idx,U,2);
      if i >= 0 then
      begin
        p := ord(Info.SigType)+1;
        Flags := piece(FItems[i],U,4);
        case cb.State of
          cbUnchecked: Flags[p] := StsChar[isUnchecked];
          cbChecked:   Flags[p] := StsChar[isChecked];
          else         Flags[p] := StsChar[isUnknown];
        end;
        FItems.SetStrPiece(i,4,Flags);
      end;
    end;
  end;
end;

procedure TSigItems.lbDrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
var
  OldRect: TRect;
  i: integer;
  cb: TORCheckBox;
  si: TSigItemType;
  DrawGrid: boolean;

begin
  DrawGrid := (Index < flb.Items.Count);
  if DrawGrid and (trim(Flb.Items[Index]) = '') and
                  (Index = (flb.Items.Count - 1)) then
    DrawGrid := FALSE;
  if DrawGrid then
    dec(Rect.Bottom);
  OldRect := Rect;
  Rect.Right := FLastValidX - 4;
  if assigned(FOldDrawItemEvent) then
    FOldDrawItemEvent(Control, Index, Rect, State)
  else
  begin
    Flb.Canvas.FillRect(Rect);
    if Index < flb.Items.Count then
      Flb.Canvas.TextRect(Rect, Rect.Left + 2, Rect.Top, FilteredString(Flb.Items[Index]));
  end;
  if DrawGrid then
  begin
    Flb.Canvas.Pen.Color := clBtnFace;
    Flb.Canvas.MoveTo(Rect.Left, Rect.Bottom);
    Flb.Canvas.LineTo(OldRect.RIght, Rect.Bottom);
  end;
  OldRect.Left := Rect.Right;
  Flb.Canvas.FillRect(OldRect);
  for i := 0 to Fcb.Count-1 do
  begin
    cb := TORCheckBox(Fcb[i]);
    if TagToItem(cb.Tag).Index = Index then
    begin
      cb.Invalidate;
      cb.Top := Rect.Top + FDy;
    end;
  end;
  if DrawGrid then
  begin
    for si := low(TSigItemType) to FLastSigItem do
    begin
      if FcbX[si] > FLastValidX then
      begin
        Flb.Canvas.MoveTo(FcbX[si] - FValidGap, Rect.Top);
        Flb.Canvas.LineTo(FcbX[si] - FValidGap, Rect.Bottom);
      end;
    end;
  end;
end;

procedure TSigItems.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  if (AComponent = Flb) and (Operation = opRemove) then
  begin
    Fcb.Clear;
    TExposedListBox(Flb).OnDrawItem := FOldDrawItemEvent;
    FOldDrawItemEvent := nil;
    Flb := nil;
  end;
end;

procedure TSigItems.EnableSettings(Index: integer; Checked: boolean);
var
  cb: TORCheckBox;
  i: integer;
  Info: TSigItemTagInfo;

begin
  if Index < 0 then exit;
  for i := 0 to Fcb.Count-1 do
  begin
    if TObject(Fcb[i]) is TORCheckBox then
    begin
      cb := TORCheckBox(Fcb[i]);
      info := TagToItem(cb.Tag);
      if info.Index = Index then
        cb.enabled := Checked;
    end;
  end;
  i := FItems.IndexOfPiece(IntToStr(Index), U, 2);
  if i >= 0 then
    FItems.SetStrPiece(i, 5, BoolChar[not Checked]);
end;

function TSigItems.OK2SaveSettings: boolean;
var
  i, Index: integer;
  s: string;

begin
  Result := TRUE;
  for i := 0 to FItems.Count-1 do
  begin
    s := FItems[i];
    Index := StrToIntDef(piece(s,U,2),-1);
    if(Index >= 0) and (piece(s,U,5) <> '1') then
    begin
      if pos(StsChar[isUnknown], piece(s, U, 4)) > 0 then
      begin
        Result := FALSE;
        break;
      end;
    end;
  end;
end;

procedure TSigItems.SaveSettings;
var
  s: string;
  i, Index: integer;
  TmpSL: TStringList;

begin
  TmpSL := TStringList.Create;
  try
    for i := 0 to FItems.Count-1 do
    begin
      s := FItems[i];
      Index := StrToIntDef(piece(s,U,2),-1);
      if(Index >= 0) and (piece(s,U,5) <> '1') then
      begin
        TmpSL.Add(Piece(s,U,1) + U + piece(s,U,4));
        FItems.SetStrPiece(i, 6, '1');
      end;
    end;
    SaveCoPayStatus(TmpSL);
  finally
    TmpSL.Free;
  end;
  i := 0;
  while i < FItems.Count do
  begin
    if Piece(FItems[i], U, 6) = '1' then
      FItems.Delete(i)
    else
      inc(i);
  end;
  Fcb.Clear;
end;

initialization
  FlagCount := ord(high(TSigItemType)) - ord(low(TSigItemType)) + 1;
  BaseFlags := StringOfChar(StsChar[isNA], FlagCount);

finalization
  FreeAndNil(uSigItems);

end.

